--WHEN ADDING:
	--REPUTATION - Add another reputation sub number to the Reputation Levels in UI.lua
	--GEAR TOKEN - Add the tier to the TierTypes list, add the new tokens in the "TOKENS" section and the Locale-enUS file, add another "elseif" to the Gear Tokens function below, add the new tier to the "Tier Tokens" in UI.lua
	--RAIDS      - Add the raid to the Raids list, add a function in the "CONSTRUCTORS" section under the "RAIDS" heading, add two elseif statements in UI.lua for the normal & heroic versions
	--DUNGEONS   - Add the dungeon to the heroic only dungeons or the other dungeon list, add a function in the "CONSTRUCTORS" section under the appropriate heading, add one or two elseif statements in UI.lua depending on if the dungeon has both a normal & heroic mode or not
	--(Other)    - Add the other things to the list they belong to

local mod = UpgradeList:NewModule("ItemLocations")
local L = LibStub("AceLocale-3.0"):GetLocale("UpgradeList")

local UPGRADELIST_VERSION = GetAddOnMetadata('UpgradeList', 'Version')

local TierTokenIDs = {}
local _, englishClass = UnitClass("player")
local GeneralItemLocations = {}
local InstanceTypes = {}
local Currency = {}
local TierTypes = {}
local ReputationLevels = {}
local Professions = {}
local Reputations = {}
local Raids = {}
local HeroicOnlyDungeons = {}
local Dungeons = {}

function UpgradeList:LoadDropdownItems()

	--Strings I Can't Extract From The Game--------------------------------------------------
	GeneralItemLocations = {
		L["Raid"],
		L["Dungeon"],
		L["Crafted"],
		L["Quest Reward"],
		L["Reputation Reward"],
		L["Purchasable"],
		L["World Drop"],
		L["Other"]
	}
		
	InstanceTypes = {
		L["Heroic"],
		L["Normal"]
	}

	Currency = {
		L["Gear Token"],
		L["Valor Points"],
		L["Justice Points"],
		L["Conquest Points"],
		L["Honor Points"]
	}
	self.db.Constants.Currency = Currency

	TierTypes = {
		L["Tier 13"],
		L["Tier 12"],
		L["Tier 11"]
	}

	ReputationLevels = {
		L["Exalted"],
		L["Revered"],
		L["Honored"],
		L["Friendly"]
	}

	--Professions--------------------------------------------------------------------------------------
	local localizedProfessionNames = {
		[1] = "78670", --Archaeology
		[2] = "2259", --Alchemy
		[3] = "2018", --Blacksmithing
		[4] = "7411", --Enchanting
		[5] = "4036", --Engineering
		[6] = "45357", --Inscription
		[7] = "25229", --Jewelcrafting
		[8] = "2108", --Leatherworking
		[9] = "3908", --Tailoring
	}

	for prof,number in pairs(localizedProfessionNames) do
		Professions[prof] = GetSpellInfo(number)
	end
	
	self.db.Constants.Professions = Professions

	--Reputations--------------------------------------------------------------------------------------
	local numCataReps = GetCategoryNumAchievements(15072)

	for i = 1, numCataReps, 1 do
		_, Reputations[i] = GetAchievementInfo(15072, i)
		_, self.db.Constants.Reputations[i] = GetAchievementInfo(15072, i)
	end

	UpgradeList:PassRepsToUI(Reputations)

	--Instances----------------------------------------------------------------------------------------
	--Add new instances to the top of their respective lists with the form - {id, "", {}}
	Raids = {
		{187, "", {}}, --Dragon Soul
		{78, "", {}}, --Firelands
		{73, "", {}}, --Blackwing Descent
		{72, "", {}}, --Bastion of Twilight
		{74, "", {}}, --Throne of the Four Winds
		{75, "", {}}, --Baradin Hold
	}
	
	LFRRaids = {
		{187, "", {}}, --Dragon Soul
	}

	HeroicOnlyDungeons = {
		{186, "", {}}, --Hour of Twilight
		{185, "", {}}, --Well of Eternity
		{184, "", {}}, --End Time
		{77, "", {}}, --Zul'Aman
		{76, "", {}}, --Zul'Gurub
		{63, "", {}}, --Deadmines
		{64, "", {}}, --Shadowfang Keep
	}

	Dungeons = {
		{66, "", {}}, --Blackrock Caverns
		{71, "", {}}, --Grim Batol
		{70, "", {}}, --Halls of Origination
		{69, "", {}}, --Lost City of the Tol'vir
		{67, "", {}}, --The Stonecore
		{65, "", {}}, --Throne of the Tides
		{68, "", {}}, --Vortex Pinnacle
	}

	for i = 1, #Raids, 1 do --for i = #Raids, 1, -1 do
		Raids[i][2] = EJ_GetInstanceInfo(Raids[i][1])
	end
	self.db.Constants.Raids = Raids
	
	for i = 1, #LFRRaids, 1 do --for i = #LFRRaids, 1, -1 do
		LFRRaids[i][2] = EJ_GetInstanceInfo(LFRRaids[i][1])
	end
	self.db.Constants.LFRRaids = LFRRaids

	for i = 1, #HeroicOnlyDungeons, 1 do --for i = #HeroicOnlyDungeons, 1, -1 do
		HeroicOnlyDungeons[i][2] = EJ_GetInstanceInfo(HeroicOnlyDungeons[i][1])
	end
	self.db.Constants.HeroicOnlyDungeons = HeroicOnlyDungeons

	for i = 1, #Dungeons, 1 do --for i = #Dungeons, 1, -1 do
		Dungeons[i][2] = EJ_GetInstanceInfo(Dungeons[i][1])
	end
	self.db.Constants.Dungeons = Dungeons

	--Bosses-------------------------------------------------------------------------------------------
	--Shouldn't ever need to change anything in here
	for i = 1, #Raids, 1 do
		local bossCounter = 1
		EJ_SelectInstance(Raids[i][1])
		EJ_SetDifficulty(4) --picks up any heroic only bosses like Sinestra
		repeat
			Raids[i][3][bossCounter] = EJ_GetEncounterInfoByIndex(bossCounter)
			bossCounter = bossCounter + 1
		until EJ_GetEncounterInfoByIndex(bossCounter) == nil
	end
	
	for i = 1, #LFRRaids, 1 do
		local bossCounter = 1
		EJ_SelectInstance(LFRRaids[i][1])
		EJ_SetDifficulty(4)
		repeat
			LFRRaids[i][3][bossCounter] = EJ_GetEncounterInfoByIndex(bossCounter)
			bossCounter = bossCounter + 1
		until EJ_GetEncounterInfoByIndex(bossCounter) == nil
	end

	for i = 1, #HeroicOnlyDungeons, 1 do
		local bossCounter = 1
		EJ_SelectInstance(HeroicOnlyDungeons[i][1])
		repeat
			HeroicOnlyDungeons[i][3][bossCounter] = EJ_GetEncounterInfoByIndex(bossCounter)
			bossCounter = bossCounter + 1
		until EJ_GetEncounterInfoByIndex(bossCounter) == nil
	end

	for i = 1, #Dungeons, 1 do
		local bossCounter = 1
		EJ_SelectInstance(Dungeons[i][1])
		repeat
			Dungeons[i][3][bossCounter] = EJ_GetEncounterInfoByIndex(bossCounter)
			bossCounter = bossCounter + 1
		until EJ_GetEncounterInfoByIndex(bossCounter) == nil
	end
	
	EJ_SetDifficulty(1) --Sets the difficulty back to normal 10 mans since that's its default
	
	--Tokens-------------------------------------------------------------------------------------------
	--Format: [##] = {"token_number", "token_name", instance, boss, "difficulty"

	if (englishClass == "DEATHKNIGHT") or (englishClass == "DRUID") or (englishClass == "MAGE") or (englishClass == "ROGUE") then --Vanquisher
		TierTokenIDs = {
			--Tier 11
			[1] = {"65002", "", Raids[3][2], Raids[3][3][6], L["Heroic"]}, --Crown
			[2] = {"65089", "", Raids[4][2], Raids[4][3][4], L["Heroic"]}, --Shoulders
			[3] = {"67425", "", Raids[4][2], Raids[4][3][1], L["Heroic"]}, --Chest
			[4] = {"67431", "", Raids[3][2], Raids[3][3][2], L["Heroic"]}, --Gauntlets
			[5] = {"67426", "", Raids[3][2], Raids[3][3][5], L["Heroic"]}, --Leggings
			[6] = {"63682", "", Raids[3][2], Raids[3][3][6], L["Normal"]}, --Helm
			[7] = {"64314", "", Raids[4][2], Raids[4][3][4], L["Normal"]}, --Mantle
			--Tier 12
			[8] = {"71670", "", Raids[2][2], Raids[2][3][7], L["Heroic"]}, --Crown 
			[9] = {"71673", "", Raids[2][2], Raids[2][3][6], L["Heroic"]}, --Shoulders 
			[10] = {"71672", "", Raids[2][2], Raids[2][3][3], L["Heroic"]}, --Chest 
			[11] = {"71669", "", Raids[2][2], Raids[2][3][5], L["Heroic"]}, --Gauntlets 
			[12] = {"71671", "", Raids[2][2], Raids[2][3][4], L["Heroic"]}, --Leggings 
			[13] = {"71668", "", Raids[2][2], Raids[2][3][7], L["Normal"]}, --Helm 
			[14] = {"71674", "", Raids[2][2], Raids[2][3][6], L["Normal"]}, --Mantle 
			--Tier 13
			[15] = {"78852", "", Raids[1][2], Raids[1][3][6], L["Heroic"]}, --Crown
			[16] = {"78861", "", Raids[1][2], Raids[1][3][4], L["Heroic"]}, --Shoulders
			[17] = {"78849", "", Raids[1][2], Raids[1][3][5], L["Heroic"]}, --Chest
			[18] = {"78855", "", Raids[1][2], Raids[1][3][2], L["Heroic"]}, --Gauntlets
			[19] = {"78858", "", Raids[1][2], Raids[1][3][3], L["Heroic"]}, --Leggings
			[20] = {"78172", "", Raids[1][2], Raids[1][3][6], L["Normal"]}, --Crown
			[21] = {"78170", "", Raids[1][2], Raids[1][3][4], L["Normal"]}, --Shoulders
			[22] = {"78174", "", Raids[1][2], Raids[1][3][5], L["Normal"]}, --Chest
			[23] = {"78173", "", Raids[1][2], Raids[1][3][2], L["Normal"]}, --Gauntlets
			[24] = {"78171", "", Raids[1][2], Raids[1][3][3], L["Normal"]}, --Leggings
			[25] = {"78868", "", Raids[1][2], Raids[1][3][6], L["LFR"]}, --Crown
			[26] = {"78874", "", Raids[1][2], Raids[1][3][4], L["LFR"]}, --Shoulders
			[27] = {"78862", "", Raids[1][2], Raids[1][3][5], L["LFR"]}, --Chest
			[28] = {"78865", "", Raids[1][2], Raids[1][3][2], L["LFR"]}, --Gauntlets
			[29] = {"78871", "", Raids[1][2], Raids[1][3][3], L["LFR"]}, --Leggings
		}
	elseif (englishClass == "PALADIN") or (englishClass == "PRIEST") or (englishClass == "WARLOCK") then --Conqueror
		TierTokenIDs = {
			--Tier 11
			[1] = {"65001", "", Raids[3][2], Raids[3][3][6], L["Heroic"]}, --Crown 
			[2] = {"65088", "", Raids[4][2], Raids[4][3][4], L["Heroic"]}, --Shoulders 
			[3] = {"67423", "", Raids[4][2], Raids[4][3][1], L["Heroic"]}, --Chest 
			[4] = {"67429", "", Raids[3][2], Raids[3][3][2], L["Heroic"]}, --Gauntlets 
			[5] = {"67428", "", Raids[3][2], Raids[3][3][5], L["Heroic"]}, --Leggings 
			[6] = {"63683", "", Raids[3][2], Raids[3][3][6], L["Normal"]}, --Helm 
			[7] = {"64315", "", Raids[4][2], Raids[4][3][4], L["Normal"]}, --Mantle 
			--Tier 12
			[8] = {"71677", "", Raids[2][2], Raids[2][3][7], L["Heroic"]}, --Crown 
			[9] = {"71680", "", Raids[2][2], Raids[2][3][6], L["Heroic"]}, --Shoulders 
			[10] = {"71679", "", Raids[2][2], Raids[2][3][3], L["Heroic"]}, --Chest 
			[11] = {"71676", "", Raids[2][2], Raids[2][3][5], L["Heroic"]}, --Gauntlets 
			[12] = {"71678", "", Raids[2][2], Raids[2][3][4], L["Heroic"]}, --Leggings 
			[13] = {"71675", "", Raids[2][2], Raids[2][3][7], L["Normal"]}, --Helm 
			[14] = {"71681", "", Raids[2][2], Raids[2][3][6], L["Normal"]}, --Mantle 
			--Tier 13
			[15] = {"78850", "", Raids[1][2], Raids[1][3][6], L["Heroic"]}, --Crown
			[16] = {"78859", "", Raids[1][2], Raids[1][3][4], L["Heroic"]}, --Shoulders
			[17] = {"78847", "", Raids[1][2], Raids[1][3][5], L["Heroic"]}, --Chest
			[18] = {"78853", "", Raids[1][2], Raids[1][3][2], L["Heroic"]}, --Gauntlets
			[19] = {"78856", "", Raids[1][2], Raids[1][3][3], L["Heroic"]}, --Leggings
			[20] = {"78182", "", Raids[1][2], Raids[1][3][6], L["Normal"]}, --Crown
			[21] = {"78180", "", Raids[1][2], Raids[1][3][4], L["Normal"]}, --Shoulders
			[22] = {"78184", "", Raids[1][2], Raids[1][3][5], L["Normal"]}, --Chest
			[23] = {"78183", "", Raids[1][2], Raids[1][3][2], L["Normal"]}, --Gauntlets
			[24] = {"78181", "", Raids[1][2], Raids[1][3][3], L["Normal"]}, --Leggings
			[25] = {"78869", "", Raids[1][2], Raids[1][3][6], L["LFR"]}, --Crown
			[26] = {"78875", "", Raids[1][2], Raids[1][3][4], L["LFR"]}, --Shoulders
			[27] = {"78863", "", Raids[1][2], Raids[1][3][5], L["LFR"]}, --Chest
			[28] = {"78866", "", Raids[1][2], Raids[1][3][2], L["LFR"]}, --Gauntlets
			[29] = {"78872", "", Raids[1][2], Raids[1][3][3], L["LFR"]}, --Leggings
		}
	elseif (englishClass == "HUNTER") or (englishClass == "SHAMAN") or (englishClass == "WARRIOR") then --Protector
		TierTokenIDs = {
			--Tier 11
			[1] = {"65000", "", Raids[3][2], Raids[3][3][6], L["Heroic"]}, --Crown 
			[2] = {"65087", "", Raids[3][2], Raids[4][3][4], L["Heroic"]}, --Shoulders 
			[3] = {"67424", "", Raids[4][2], Raids[4][3][1], L["Heroic"]}, --Chest 
			[4] = {"67430", "", Raids[3][2], Raids[3][3][2], L["Heroic"]}, --Gauntlets 
			[5] = {"67427", "", Raids[3][2], Raids[3][3][5], L["Heroic"]}, --Leggings 
			[6] = {"63684", "", Raids[3][2], Raids[3][3][6], L["Normal"]}, --Helm 
			[7] = {"64316", "", Raids[4][2], Raids[4][3][4], L["Normal"]}, --Mantle 
			--Tier 12
			[8] = {"71684", "", Raids[2][2], Raids[2][3][7], L["Heroic"]}, --Crown 
			[9] = {"71687", "", Raids[2][2], Raids[2][3][6], L["Heroic"]}, --Shoulders 
			[10] = {"71686", "", Raids[2][2], Raids[2][3][3], L["Heroic"]}, --Chest 
			[11] = {"71683", "", Raids[2][2], Raids[2][3][5], L["Heroic"]}, --Gauntlets 
			[12] = {"71685", "", Raids[2][2], Raids[2][3][4], L["Heroic"]}, --Leggings 
			[13] = {"71682", "", Raids[2][2], Raids[2][3][7], L["Normal"]}, --Helm 
			[14] = {"71688", "", Raids[2][2], Raids[2][3][6], L["Normal"]}, --Mantle 
			--Tier 13
			[15] = {"78851", "", Raids[1][2], Raids[1][3][6], L["Heroic"]}, --Crown
			[16] = {"78860", "", Raids[1][2], Raids[1][3][4], L["Heroic"]}, --Shoulders
			[17] = {"78848", "", Raids[1][2], Raids[1][3][5], L["Heroic"]}, --Chest
			[18] = {"78854", "", Raids[1][2], Raids[1][3][2], L["Heroic"]}, --Gauntlets
			[19] = {"78857", "", Raids[1][2], Raids[1][3][3], L["Heroic"]}, --Leggings
			[20] = {"78177", "", Raids[1][2], Raids[1][3][6], L["Normal"]}, --Crown
			[21] = {"78175", "", Raids[1][2], Raids[1][3][4], L["Normal"]}, --Shoulders
			[22] = {"78179", "", Raids[1][2], Raids[1][3][5], L["Normal"]}, --Chest
			[23] = {"78178", "", Raids[1][2], Raids[1][3][2], L["Normal"]}, --Gauntlets
			[24] = {"78176", "", Raids[1][2], Raids[1][3][3], L["Normal"]}, --Leggings
			[25] = {"78870", "", Raids[1][2], Raids[1][3][6], L["LFR"]}, --Crown
			[26] = {"78876", "", Raids[1][2], Raids[1][3][4], L["LFR"]}, --Shoulders
			[27] = {"78864", "", Raids[1][2], Raids[1][3][5], L["LFR"]}, --Chest
			[28] = {"78867", "", Raids[1][2], Raids[1][3][2], L["LFR"]}, --Gauntlets
			[29] = {"78873", "", Raids[1][2], Raids[1][3][3], L["LFR"]}, --Leggings
		}
	end

	for i = 1, #TierTokenIDs, 1 do
		TierTokenIDs[i][2] = GetItemInfo(TierTokenIDs[i][1])
	end

	function UpgradeList:LoadTierTokenNames()
		for i = 1, #TierTokenIDs, 1 do
			TierTokenIDs[i][2] = GetItemInfo(TierTokenIDs[i][1])
		end
	end
end
--------------------------------------------CONSTRUCTORS--------------------------------------------
function UpgradeList:TopLevel(info, level)
	for i = 1, #GeneralItemLocations, 1 do
		info = UIDropDownMenu_CreateInfo()
		info.text = GeneralItemLocations[i]
		info.notCheckable = 1
		if info.text == L["Other"] then
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Other"])
				self.db.SelectedBoss = ""
				self.db.SelectedInstance = L["Other"]
				self.db.SelectedDifficulty = ""
				CloseDropDownMenus(1)
			end
			UIDropDownMenu_AddButton(info, level)
		else
			info.hasArrow = true
			info.value = info.text
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

function UpgradeList:RaidSubmenu(info, level)
	for i = 1, #InstanceTypes, 1 do
		info.text = InstanceTypes[i]
		info.hasArrow = true
		info.value = InstanceTypes[i] .. "Raid"
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
	info.text = "LFR"
	info.hasArrow = true
	info.value = "LFR" .. "Raid"
	info.notCheckable = 1
	UIDropDownMenu_AddButton(info, level)
end

function UpgradeList:DungeonSubmenu(info, level)
	for i = 1, #InstanceTypes, 1 do
		info.text = InstanceTypes[i]
		info.hasArrow = true
		info.value = InstanceTypes[i] .. "Dungeon"
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:CraftedSubmenu(info, level)
	for i = 1, #Professions, 1 do
		info.text = Professions[i]
		info.value = info.text
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Professions[i])
			self.db.SelectedBoss = ""
			self.db.SelectedInstance = Professions[i]
			self.db.SelectedDifficulty = ""
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:Reputations(info, level)
	for i = 1, #Reputations, 1 do
		info.text = Reputations[i]
		info.hasArrow = true
		info.value = Reputations[i]
		info.notCheckable = true
		UIDropDownMenu_AddButton(info, level)
	end
	if IsInGuild() then
		info.text = GetGuildInfo("player")
		info.hasArrow = true
		info.value = info.text
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:RepLevels(info, level, rep)
	for i = 1, #ReputationLevels, 1 do
		info.text = ReputationLevels[i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, rep)
			self.db.SelectedBoss = ReputationLevels[i]
			self.db.SelectedInstance = rep
			self.db.SelectedDifficulty = ""
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:Purchasable(info, level)
	for i = 1, #Currency, 1 do
		info.text = Currency[i]
		info.value = info.text
		info.notCheckable = 1
		if Currency[i] == L["Gear Token"] then
			info.hasArrow = true
		else
			info.hasArrow = false
			function info.func()
				CloseDropDownMenus(1)
				UIDropDownMenu_SetSelectedName(ULLocationDropDown_Button, Currency[i])
				self.db.UI.PointInput:Show()
				self.db.UI.PointInput.box:SetFocus()
				self.db.UI.MainFrame:EnableMouse(false)
				self.db.UI.CloseButton:EnableMouse(false)
				self.db.UI.ToggleButton:EnableMouse(false)
				self.db.UI.ViewPrimaryCheck:EnableMouse(false)
				self.db.UI.ViewSecondaryCheck:EnableMouse(false)
				self.db.UI.OptionsFrame:EnableMouse(false)
				self.db.UI.ItemEntryBox:EnableMouse(false)
				ULLocationDropDown_Button:EnableMouse(false)
				self.db.UI.NoteEntryBox:EnableMouse(false)
				self.db.UI.PrimarySpecRadioButton:EnableMouse(false)
				self.db.UI.SecondarySpecRadioButton:EnableMouse(false)
				self.db.UI.PriorityField:EnableMouse(false)
				self.db.UI.AddButton:EnableMouse(false)
				self.db.UI.Moving.tofield:EnableMouse(false)
				self.db.UI.Moving.fromfield:EnableMouse(false)
				self.db.UI.Moving.button:EnableMouse(false)
				self.db.UI.DeletingSingle.button:EnableMouse(false)
				self.db.UI.DeletingSingle.field:EnableMouse(false)
				self.db.UI.DeletingMultiple.button:EnableMouse(false)
				self.db.UI.DeletingMultiple.multifromfield:EnableMouse(false)
				self.db.UI.DeletingMultiple.multitofield:EnableMouse(false)
			end
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

--All raids have a 1 in the front, Heroic Only Dungeons have a 2 in the front, Heroic Dungeons have a 3 in the front, and Normal Dungeons have a 4 in the front
function UpgradeList:HeroicRaids(info, level)
	for i = 1, #Raids - 1, 1 do --#Raids - 1 fixes Encounter Journal's problem of listing Baradin Hold as having a heroic mode
		info.text = Raids[i][2]
		info.value = "1 " .. i .. " Heroic"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:NormalRaids(info, level)
	for i = 1, #Raids, 1 do
		info.text = Raids[i][2]
		info.value = "1 " .. i .. " Normal"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:LFRRaids(info, level)
	for i = 1, #LFRRaids, 1 do
		info.text = LFRRaids[i][2]
		info.value = "1 " .. i .. " LFR"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:HeroicDungeons(info, level)
	for i = 1, #HeroicOnlyDungeons, 1 do
		info.text = HeroicOnlyDungeons[i][2]
		info.value = "2 " .. i .. " Heroic"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
	for i = 1, #Dungeons, 1 do
		info.text = Dungeons[i][2]
		info.value = "3 " .. i .. " Heroic"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:NormalDungeons(info, level)
	for i = 1, #Dungeons, 1 do
		info.text = Dungeons[i][2]
		info.value = "4 " .. i .. " Normal"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:TierDifficulty(info, level)
	for i = 1, #InstanceTypes, 1 do
		info.text = InstanceTypes[i]
		info.hasArrow = true
		info.value = InstanceTypes[i] .. "Tier"
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
	info.text = L["LFR"]
	info.hasArrow = true
	info.value = "LFR" .. "Tier"
	info.notCheckable = 1
	UIDropDownMenu_AddButton(info, level)
end

function UpgradeList:TierSelection(info, level, difficulty)
	if difficulty ~= "LFRTier" then
		for i = 1, #TierTypes, 1 do
			info.text = TierTypes[i]
			info.value = TierTypes[i]
			info.hasArrow = true
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)			
		end
	else
		info.text = TierTypes[1]
		info.value = TierTypes[1]
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:GearTokens(info, level, tier, difficulty)
	--difficulty can be HeroicTier, NormalTier, or LFRTier
	local _, englishClass = UnitClass("player")
	if tier == L["Tier 11"] then
		if difficulty == "HeroicTier" then
			for i = 1, 5, 1 do --Tier 11 tokens
				info.text = TierTokenIDs[i][2]
				info.notCheckable = true
				function info.func()
					UIDropDownMenu_SetText(ULLocationDropDown_Button, TierTokenIDs[i][2])
					self.db.SelectedBoss = TierTokenIDs[i][4]
					self.db.SelectedInstance = TierTokenIDs[i][3]
					self.db.SelectedDifficulty = TierTokenIDs[i][5]
					CloseDropDownMenus(1)
				end
				UIDropDownMenu_AddButton(info, level)
			end
		elseif difficulty == "NormalTier" then
			for i = 6, 7, 1 do --Tier 11 tokens
				info.text = TierTokenIDs[i][2]
				info.notCheckable = true
				function info.func()
					UIDropDownMenu_SetText(ULLocationDropDown_Button, TierTokenIDs[i][2])
					self.db.SelectedBoss = TierTokenIDs[i][4]
					self.db.SelectedInstance = TierTokenIDs[i][3]
					self.db.SelectedDifficulty = TierTokenIDs[i][5]
					CloseDropDownMenus(1)
				end
				UIDropDownMenu_AddButton(info, level)
			end
		end
	elseif tier == L["Tier 12"] then
		if difficulty == "HeroicTier" then
			for i = 8, 12, 1 do --Tier 12 tokens
				info.text = TierTokenIDs[i][2]
				info.notCheckable = true
				function info.func()
					UIDropDownMenu_SetText(ULLocationDropDown_Button, TierTokenIDs[i][2])
					self.db.SelectedBoss = TierTokenIDs[i][4]
					self.db.SelectedInstance = TierTokenIDs[i][3]
					self.db.SelectedDifficulty = TierTokenIDs[i][5]
					CloseDropDownMenus(1)
				end
				UIDropDownMenu_AddButton(info, level)
			end
		elseif difficulty == "NormalTier" then
			for i = 13, 14, 1 do --Tier 12 tokens
				info.text = TierTokenIDs[i][2]
				info.notCheckable = true
				function info.func()
					UIDropDownMenu_SetText(ULLocationDropDown_Button, TierTokenIDs[i][2])
					self.db.SelectedBoss = TierTokenIDs[i][4]
					self.db.SelectedInstance = TierTokenIDs[i][3]
					self.db.SelectedDifficulty = TierTokenIDs[i][5]
					CloseDropDownMenus(1)
				end
				UIDropDownMenu_AddButton(info, level)
			end
		end
	elseif tier == L["Tier 13"] then
		if difficulty == "HeroicTier" then
			for i = 15, 19, 1 do --Heroic Tier 13 tokens
				info.text = TierTokenIDs[i][2]
				info.notCheckable = true
				function info.func()
					UIDropDownMenu_SetText(ULLocationDropDown_Button, TierTokenIDs[i][2])
					self.db.SelectedBoss = TierTokenIDs[i][4]
					self.db.SelectedInstance = TierTokenIDs[i][3]
					self.db.SelectedDifficulty = TierTokenIDs[i][5]
					CloseDropDownMenus(1)
				end
				UIDropDownMenu_AddButton(info, level)
			end
		elseif difficulty == "NormalTier" then
			for i = 20, 24, 1 do --Normal Tier 13 tokens
				info.text = TierTokenIDs[i][2]
				info.notCheckable = true
				function info.func()
					UIDropDownMenu_SetText(ULLocationDropDown_Button, TierTokenIDs[i][2])
					self.db.SelectedBoss = TierTokenIDs[i][4]
					self.db.SelectedInstance = TierTokenIDs[i][3]
					self.db.SelectedDifficulty = TierTokenIDs[i][5]
					CloseDropDownMenus(1)
				end
				UIDropDownMenu_AddButton(info, level)
			end
		elseif difficulty == "LFRTier" then
			for i = 25, 29, 1 do --LFR Tier 13 tokens
				info.text = TierTokenIDs[i][2]
				info.notCheckable = true
				function info.func()
					UIDropDownMenu_SetText(ULLocationDropDown_Button, TierTokenIDs[i][2])
					self.db.SelectedBoss = TierTokenIDs[i][4]
					self.db.SelectedInstance = TierTokenIDs[i][3]
					self.db.SelectedDifficulty = TierTokenIDs[i][5]
					CloseDropDownMenus(1)
				end
				UIDropDownMenu_AddButton(info, level)
			end
		end
	end
end

--RAIDS
--When adding a new raid, the first number after "Raids" will be incremented - Raids[+1][3][i]
function UpgradeList:DragonSoul(info, level, difficulty)
	if difficulty ~= L["LFR"] then
		for i = 1, #Raids[6][3], 1 do
			info.text = Raids[6][3][i]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, Raids[6][3][i])
				self.db.SelectedBoss = Raids[6][3][i]
				self.db.SelectedInstance = Raids[6][2]
				self.db.SelectedDifficulty = difficulty
				CloseDropDownMenus(1)
			end
			UIDropDownMenu_AddButton(info, level)
		end
	else
		for i = 1, #LFRRaids[1][3], 1 do
			info.text = LFRRaids[1][3][i]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, Raids[1][3][i])
				self.db.SelectedBoss = Raids[1][3][i]
				self.db.SelectedInstance = Raids[1][2]
				self.db.SelectedDifficulty = difficulty
				CloseDropDownMenus(1)
			end
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

function UpgradeList:Firelands(info, level, difficulty)
	for i = 1, #Raids[5][3], 1 do
		info.text = Raids[5][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Raids[5][3][i])
			self.db.SelectedBoss = Raids[5][3][i]
			self.db.SelectedInstance = Raids[5][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:BaradinHold(info, level, difficulty)
	for i = 1, #Raids[1][3], 1 do
		info.text = Raids[1][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Raids[1][3][i])
			self.db.SelectedBoss = Raids[1][3][i]
			self.db.SelectedInstance = Raids[1][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:BastionOfTwilight(info, level, difficulty)
	if difficulty == L["Normal"] then
		for i = 1, #Raids[3][3] - 1, 1 do
			info.text = Raids[3][3][i]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, Raids[3][3][i])
				self.db.SelectedBoss = Raids[3][3][i]
				self.db.SelectedInstance = Raids[3][2]
				self.db.SelectedDifficulty = difficulty
				CloseDropDownMenus(1)
			end
			UIDropDownMenu_AddButton(info, level)
		end
	else
		for i = 1, #Raids[3][3], 1 do
			info.text = Raids[3][3][i]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, Raids[3][3][i])
				self.db.SelectedBoss = Raids[3][3][i]
				self.db.SelectedInstance = Raids[3][2]
				self.db.SelectedDifficulty = difficulty
				CloseDropDownMenus(1)
			end
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

function UpgradeList:BlackwingDescent(info, level, difficulty)
	for i = 1, #Raids[4][3], 1 do
		info.text = Raids[4][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Raids[4][3][i])
			self.db.SelectedBoss = Raids[4][3][i]
			self.db.SelectedInstance = Raids[4][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:ThroneOfTheFourWinds(info, level, difficulty)
	for i = 1, #Raids[2][3], 1 do
		info.text = Raids[2][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Raids[2][3][i])
			self.db.SelectedBoss = Raids[2][3][i]
			self.db.SelectedInstance = Raids[2][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

--HEROIC ONLY DUNGEONS
--When adding a new heroic only dungeon, the first number after "HeroicOnlyDungeons" will be incremented - HeroicOnlyDungeons[+1][3][i]
function UpgradeList:HourOfTwilight(info, level)
	for i = 1, #HeroicOnlyDungeons[7][3], 1 do
		info.text = HeroicOnlyDungeons[7][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, HeroicOnlyDungeons[7][3][i])
			self.db.SelectedBoss = HeroicOnlyDungeons[7][3][i]
			self.db.SelectedInstance = HeroicOnlyDungeons[7][2]
			self.db.SelectedDifficulty = L["Heroic"]
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:WellOfEternity(info, level)
	for i = 1, #HeroicOnlyDungeons[6][3], 1 do
		info.text = HeroicOnlyDungeons[6][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, HeroicOnlyDungeons[6][3][i])
			self.db.SelectedBoss = HeroicOnlyDungeons[6][3][i]
			self.db.SelectedInstance = HeroicOnlyDungeons[6][2]
			self.db.SelectedDifficulty = L["Heroic"]
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:EndTime(info, level)
	for i = 1, #HeroicOnlyDungeons[5][3], 1 do
		info.text = HeroicOnlyDungeons[5][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, HeroicOnlyDungeons[5][3][i])
			self.db.SelectedBoss = HeroicOnlyDungeons[5][3][i]
			self.db.SelectedInstance = HeroicOnlyDungeons[5][2]
			self.db.SelectedDifficulty = L["Heroic"]
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:ZulAman(info, level)
	for i = 1, #HeroicOnlyDungeons[4][3], 1 do
		info.text = HeroicOnlyDungeons[4][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, HeroicOnlyDungeons[4][3][i])
			self.db.SelectedBoss = HeroicOnlyDungeons[4][3][i]
			self.db.SelectedInstance = HeroicOnlyDungeons[4][2]
			self.db.SelectedDifficulty = L["Heroic"]
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:ZulGurub(info, level)
	for i = 1, #HeroicOnlyDungeons[3][3], 1 do
		info.text = HeroicOnlyDungeons[3][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, HeroicOnlyDungeons[3][3][i])
			self.db.SelectedBoss = HeroicOnlyDungeons[3][3][i]
			self.db.SelectedInstance = HeroicOnlyDungeons[3][2]
			self.db.SelectedDifficulty = L["Heroic"]
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:Deadmines(info, level)
	for i = 1, #HeroicOnlyDungeons[2][3], 1 do
		info.text = HeroicOnlyDungeons[2][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, HeroicOnlyDungeons[2][3][i])
			self.db.SelectedBoss = HeroicOnlyDungeons[2][3][i]
			self.db.SelectedInstance = HeroicOnlyDungeons[2][2]
			self.db.SelectedDifficulty = L["Heroic"]
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:ShadowfangKeep(info, level)
	for i = 1, #HeroicOnlyDungeons[1][3], 1 do
		info.text = HeroicOnlyDungeons[1][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, HeroicOnlyDungeons[1][3][i])
			self.db.SelectedBoss = HeroicOnlyDungeons[1][3][i]
			self.db.SelectedInstance = HeroicOnlyDungeons[1][2]
			self.db.SelectedDifficulty = L["Heroic"]
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

--DUNGEONS WITH NORMAL & HEROIC MODES
--When adding a new dungeon, the first number after "Dungeons" will be incremented - Dungeons[+1][3][i]
function UpgradeList:ThroneOfTides(info, level, difficulty)
	for i = 1, #Dungeons[2][3], 1 do
		info.text = Dungeons[2][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Dungeons[2][3][i])
			self.db.SelectedBoss = Dungeons[2][3][i]
			self.db.SelectedInstance = Dungeons[2][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:BlackrockCaverns(info, level, difficulty)
	for i = 1, #Dungeons[7][3], 1 do
		info.text = Dungeons[7][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Dungeons[7][3][i])
			self.db.SelectedBoss = Dungeons[7][3][i]
			self.db.SelectedInstance = Dungeons[7][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:Stonecore(info, level, difficulty)
	for i = 1, #Dungeons[3][3], 1 do
		info.text = Dungeons[3][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Dungeons[3][3][i])
			self.db.SelectedBoss = Dungeons[3][3][i]
			self.db.SelectedInstance = Dungeons[3][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:VortexPinnacle(info, level, difficulty)
	for i = 1, #Dungeons[1][3], 1 do
		info.text = Dungeons[1][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Dungeons[1][3][i])
			self.db.SelectedBoss = Dungeons[1][3][i]
			self.db.SelectedInstance = Dungeons[1][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:LostCityOfTolvir(info, level, difficulty)
	for i = 1, #Dungeons[4][3], 1 do
		info.text = Dungeons[4][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Dungeons[4][3][i])
			self.db.SelectedBoss = Dungeons[4][3][i]
			self.db.SelectedInstance = Dungeons[4][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:GrimBatol(info, level, difficulty)
	for i = 1, #Dungeons[6][3], 1 do
		info.text = Dungeons[6][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Dungeons[6][3][i])
			self.db.SelectedBoss = Dungeons[6][3][i]
			self.db.SelectedInstance = Dungeons[6][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:HallsOfOrigination(info, level, difficulty)
	for i = 1, #Dungeons[5][3], 1 do
		info.text = Dungeons[5][3][i]
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(ULLocationDropDown_Button, Dungeons[5][3][i])
			self.db.SelectedBoss = Dungeons[5][3][i]
			self.db.SelectedInstance = Dungeons[5][2]
			self.db.SelectedDifficulty = difficulty
			CloseDropDownMenus(1)
		end
		UIDropDownMenu_AddButton(info, level)
	end
end